#include "model.h"

#ifdef HAVE_QT5
#include <QtWidgets>
#include <QtConcurrent/QtConcurrent>
#else
#include <QtGui>
#endif
#include "enums.h"
#include "stringroutines.h"
#include "application.h"
#include "undoactions.h"
#include "tagsreaderwriter.h"
#include "tab.h"
#include "tagrender.h"
#include <iostream>

Model::Model(QObject *parent) :
    QObject(parent)
{

}

void Model::selectAll()
{
    indexes.clear();
    for (int i=0; i<tags.size(); ++i)
        indexes << i;
}

bool Model::isEmpty() const
{
    return tags.isEmpty();
}

bool Model::hasSelection() const
{
    return !indexes.isEmpty();
}

bool Model::selectionSaved() const
{
    for (int i=0; i<indexes.size(); ++i) {
        if (tags.at(indexes.at(i)).wasChanged())
            return false;
    }
    return true;
}

void Model::setSelection(const QList<int> &selection)
{
    indexes=selection;
}

int Model::totalLength() const
{
    int totalLength=0;
    Q_FOREACH(int i, indexes) totalLength += tags.at(i).length();
    return totalLength;
}

QList<Tag> Model::selectedFiles()
{
    QList<Tag> files;
    Q_FOREACH (int i,indexes)
        files << tags.at(i);
    return files;
}

bool Model::allSaved() const
{
    for (int i=0; i<tags.count(); ++i) {
        if (tags[i].wasChanged())
            return false;
    }
    return true;
}

bool Model::allSelectedFilesAreMP3() const
{
    if (indexes.isEmpty()) return false;
    Q_FOREACH (int i, indexes) {
        if (tags.at(i).fileType()!=Tag::MP3_FILE) return false;
    }
    return true;
}

bool Model::imagesAreSame() const
{
    QByteArray h=tags.at(indexes.at(0)).image().pixmap();
    for (int i=1; i<indexes.size(); ++i) {
        QByteArray hh=tags.at(indexes.at(i)).image().pixmap();
        if (hh!=h) {
            return false;
        }
    }
    return true;
}

int Model::size() const
{
    return tags.size();
}

Tag& Model::fileAt(int index)
{
    return tags[index];
}

Tag &Model::fileAtSelection(int index)
{
    return tags[indexes.at(index)];
}

Tag readTags(const QString &fileName)
{
    Tag tag = Tag(fileName);
    TagsReaderWriter t(&tag);
    t.readTags();

    return tag;
}

QList<Tag> Model::addFiles(const QStringList &filesToAdd)
{
    QList<Tag> tagsToAdd = QtConcurrent::blockingMapped(filesToAdd, readTags);
    addFiles(tagsToAdd);
    return tagsToAdd;
}

void Model::addFiles(const QList<Tag> &tagsToAdd)
{
    tags.append(tagsToAdd);
}

void Model::delFiles()
{
    if (tags.size()==indexes.size()) {
        tags.clear();
    }
    else {
        for (int i=indexes.size()-1; i>=0; --i)
            tags.removeAt(indexes.at(i));
    }
    indexes.clear();
}

bool wasChanged(const Tag &tag)
{
    return tag.wasChanged();
}

void Model::save(bool silently)
{
    //find out what files to save
    QList<bool> wasChangedList = QtConcurrent::blockingMapped(tags, wasChanged);
    QList<int> toSave;
    for (int i=0; i<tags.size(); ++i)
        if (wasChangedList.at(i)) toSave << i;
    saveFiles(toSave, silently);
    if (!silently) {
        for (int i=0; i<tags.size(); ++i) {
            if (wasChangedList.at(i) != tags.at(i).wasChanged())
                emit fileChanged(i, tags.at(i).wasChanged());
        }
    }
}

bool Model::saveAt(int index)
{
    if (index<0 || index>=tags.count()) return false;

    if (tags.at(index).wasChanged()) {
        saveFiles(QList<int>()<<index);
        emit fileChanged(index, tags.at(index).wasChanged());
        return true;
    }
    return false;
}

void Model::forceSelectionToSave()
{
    Q_FOREACH (int i, indexes)
        tags[i].setChanged(true);
}

void Model::saveFiles(const QList<int> &toSave, bool silently)
{
    int count = toSave.count();
    QStringList errors;
    QProgressDialog *progress=0;
    if (!silently) {
        progress=new QProgressDialog(tr("Writing tags..."), tr("Abort"), 0, count);
        progress->setWindowTitle(tr("Qoobar"));
        progress->setWindowModality(Qt::WindowModal);
    }
    for (int i=0; i<count; ++i) {
        qApp->processEvents();
        //write tags
        QString error;

        TagsReaderWriter trw(&tags[toSave.at(i)]);
        if (!trw.writeTags()) {
            QString f=tags.at(toSave.at(i)).fullFileName();
            QFileInfo info(f);
            if (!info.exists()) error = tr("%1 does not exist").arg(f);
            else if (!info.isFile()) error = tr("%1 is not a file").arg(f);
            else if (!info.isWritable()) error = tr("%1 is read-only").arg(f);
            else error = f;
        }
        if (!error.isEmpty())
            errors << error;

        if (!silently) {
            progress->setValue(i);
            if (progress->wasCanceled()) {
                break;
            }
        }
    }
    if (!silently) {
        progress->setValue(count);
        progress->deleteLater();
    }

    if (!errors.isEmpty())
        emit message(MT_WARNING, tr("Cannot write tags to files:\n%1")
                     .arg(errors.join(QChar('\n'))));
}


QStringList Model::values(int tagID) const
{
    QStringList list;
    Q_FOREACH(const int &i, indexes)
        list << tags[i].tag(tagID);
    return list;
}

QStringList Model::values(const QString &tagID) const
{
    QStringList list;
    Q_FOREACH(const int &i,indexes)
        list << tags[i].userTag(tagID);
    return list;
}

QString Model::value(int index, int tagID) const
{
    return tags.at(index).tag(tagID);
}

QString Model::value(int index, const QString &tagID) const
{
    return tags.at(index).userTag(tagID);
}

QStringList Model::fileNames() const
{
    QStringList list;
    Q_FOREACH(const int &i,indexes)
        list << tags.at(i).fullFileName();
    return list;
}

QStringList Model::userTagsIds() const
{
    QStringList otherid;
    Q_FOREACH (int i, indexes)
        otherid.append(tags.at(i).userTagsKeys());
    qSort(otherid);
    otherid.removeDuplicates();
    return otherid;
}

void Model::rename(const QStringList &newFileNames)
{
    QStringList notRenamedFiles;
    QString messageStr;

    for (int i=0; i<indexes.size(); ++i) {
        QString oldFileName = fileAtSelection(i).fullFileName();
        QString newFileName = newFileNames.at(i);
        if (oldFileName == newFileName) continue;
        int index = newFileName.lastIndexOf('/');
        QString path=newFileName.left(index);
        QDir dir(path);
        bool created=true;
        bool processed=true;
        if (!dir.exists()) created = QDir().mkpath(path);
        if (created)
            processed = (App->renamingOperation==0 ? QFile::rename(oldFileName,newFileName) : QFile::copy(oldFileName,newFileName));

        if (!created || !processed)
            notRenamedFiles.append(oldFileName);
        else {
            tags[indexes.at(i)].setFile(newFileName);
            emit fileNameChanged(indexes.at(i), tags.at(indexes.at(i)).fileNameExt());
        }
        if (App->renamingOperation==0 && App->removeFolder) {
            QFileInfo fi(oldFileName);
            QDir().rmdir(fi.absolutePath());
        }
    }
    if (!notRenamedFiles.isEmpty()) {
        if (App->renamingOperation==0) messageStr = tr("Unable to rename following files:\n");
        else messageStr = tr("Unable to copy following files:\n");
        emit message(MT_WARNING, messageStr + notRenamedFiles.join(QSL("\n")));
    }
    else {
        if (App->renamingOperation==0)
            messageStr = tr("All files were successfully renamed");
        else messageStr = tr("All files were successfully copied");
        messageStr=QString("<font color=green>%1</font>").arg(messageStr);
        emit message(MT_INFORMATION, messageStr);
    }
}

void Model::setImage(const CoverImage &img)
{
    Q_FOREACH (const int &i, indexes) {
        setNewImage(i, img);
    }
}

void Model::setImageDescription(const QString &description)
{
    Q_FOREACH (int i, indexes) {
        tags[i].setImageDescription(description);
    }
}

void Model::setImageType(int type)
{
    Q_FOREACH (int i, indexes) {
        tags[i].setImageType(type);
    }
}

bool Model::writeImage(QString dirName)
{
    bool dirIsEmpty = dirName.isEmpty();
    if (!dirIsEmpty) {
        if (dirName.startsWith(QLS("~"))) {
            dirName.replace(0,1,QDir::homePath());
        }

        QDir dir(dirName);
        if (!dir.exists()) {
            if (!dir.mkdir(dirName))  {
                emit message(MT_ERROR, QString("Cannot create directory \"%1\" to write images").arg(dirName));
                return false;
            }
        }
        dirName = dir.absolutePath();
    }

    Q_FOREACH (int i, indexes) {
        CoverImage img = tags.at(i).image();
        if (img.isEmpty()) continue;

        if (dirIsEmpty) dirName = tags.at(i).filePath();
        QString fileName = tags.at(i).fileNameExt();
        QString ext = QSL(".jpg");
        if (img.mimetype()=="image/png") ext=QSL(".png");
        fileName.append(ext);
        fileName = QString("%1/%2").arg(dirName).arg(fileName);

        QImage image;
        image.loadFromData(img.pixmap());
        if (!image.save(fileName)) {
            emit message(MT_ERROR, QString("Cannot write image into %1").arg(fileName));
        }
    }
    return true;
}

bool Model::setNewImage(int ind,const CoverImage &image)
{
    bool res=tags[ind].wasChanged();
    tags[ind].setImage(image);
    if (res!=tags[ind].wasChanged()) {
        emit fileChanged(ind, tags[ind].wasChanged());
    }
    emit imageChanged(ind, tags[ind].imageIsEmpty());
    return res;
}

void Model::setOldImage(int ind,const CoverImage &image, bool status)
{
    bool res=tags[ind].wasChanged();
    tags[ind].setImage(image);
    tags[ind].setChanged(status);
    if (res!=status) {
        emit fileChanged(ind, status);
    }
    emit imageChanged(ind, tags[ind].imageIsEmpty());
}

QList<int> computeIndexes(QList<int> notYetMoved, bool up, int totalSize)
{
    QList<int> moved;
    if (up) {
        while (notYetMoved.size()>0) {
            int j=notYetMoved.takeFirst();
            if (j==0 || moved.contains(j-1)) moved << j;
            else moved << j-1;
        }
    }
    else {
        int lastIndex = totalSize;
        while (notYetMoved.size()>0) {
            int j=notYetMoved.takeLast();
            if (j==lastIndex || moved.contains(j+1)) moved.prepend(j);
            else moved.prepend(j+1);
        }
    }
    return moved;
}

void Model::move(bool up)
{
    QList<int> newIndexes = computeIndexes(indexes, up, tags.size());
    int i=up?0:indexes.size()-1;
    while (1) {
        tags.move(indexes.at(i),newIndexes.at(i));
        if ((up && i==indexes.size()-1) || (!up && i==0)) break;
        i=up?i+1:i-1;
    }
    indexes=newIndexes;
}

void Model::rereadTags(bool all, bool force)
{
    if (!hasSelection() && !all) return;

    QList<int> inds;
    QList<Tag> newTags;
    int count=size();

    for (int i=0; i<count; ++i) {
        if (all || indexes.contains(i)) {
            Tag tag(tags.at(i).fullFileName());
            TagsReaderWriter t(&tag);
            t.readTags();
            if (tags.at(i)==tag && !force) {
                continue;
            }
            newTags << tag;
            inds << i;
        }
    }
    setTags(inds, newTags);
}

void Model::setTags(const QList<int> &inds, const QList<Tag> &newTags)
{
    for (int i=0; i<inds.count(); ++i) {
        bool status = tags.at(inds.at(i)).wasChanged();
        bool newStatus = newTags.at(i).wasChanged();
        tags.replace(inds.at(i),newTags.at(i));
        emit fileChanged(inds.at(i), newTags.at(i),QList<int>()); //update whole row
        if (newStatus != status)
            emit fileChanged(inds.at(i), newStatus);
    }
}

void Model::setRow(int tagID, const QString &newValue)
{
    if (tagID>=TAGSCOUNT) return;
    QStringList newValues = tagsByPattern(tagID, newValue);
    for (int i=0; i<indexes.size(); ++i) {
        setTag(indexes.at(i), tagID, newValues.at(i));
    }
}

void Model::setRow(const QString &stringID, const QString &newValue)
{
    QStringList newValues = tagsByPattern(-1, newValue);
    for (int i=0; i<indexes.size(); ++i) {
        setTag(indexes.at(i), stringID, newValues.at(i));
    }
}

void Model::setTag(int index, int tagID, const QString &newValue)
{

    tags[index].setTag(tagID, newValue);
}

void Model::setTag(int index, const QString &tagID, const QString &newValue)
{
    tags[index].setUserTag(tagID, newValue);
}

void Model::removeAllTags()
{
    Q_FOREACH (int i, indexes) {
        tags[i].removeAllTags();
    }
}

void Model::removeUserTags()
{
    Q_FOREACH (int i, indexes) {
        tags[i].removeUserTags();
    }
}

void Model::removeImage()
{
    setImage(CoverImage());
}

bool Model::newTagsEqual(const QStringList &newTags, int tagID, const QString &stringID)
{
    for (int i=0; i<newTags.size(); ++i) {
        QString oldTag;
        if (tagID<TAGSCOUNT)
            oldTag = tags.at(indexes.at(i)).tag(tagID);
        else
            oldTag = tags.at(indexes.at(i)).userTag(stringID);
        if (newTags.at(i) != oldTag) return false;
    }
    return true;
}

QStringList Model::tagsByPattern(int tagID, const QString &pattern)
{
    if (pattern.isEmpty()) return QVector<QString>(indexes.size(), pattern).toList();

    QStringList newValues;
    bool ook;
    bool n = pattern.startsWith('0');
    int x=pattern.toInt(&ook);
    if (ook && tagID==TRACKNUMBER) {
        for (int i=x; i<x+indexes.size(); ++i)
            newValues << (n?QString::number(i).rightJustified(2,QChar('0')):QString::number(i));
    }
    else {
        if (!pattern.contains(QRegExp(QSL("[\\[{%]"))))
            newValues=QVector<QString>(indexes.size(), pattern).toList();
        else {
            SimpleTagsRenderer renderer(this);
            renderer.setPattern(pattern);
            newValues = renderer.getNewTags();
        }
    }
    return newValues;
}
