#include "searchpanel.h"

#ifdef HAVE_QT5
#include <QtWidgets>
#else
#include <QtGui>
#endif

#include "enums.h"
#include "clearlineedit.h"

SearchPanel::SearchPanel(QWidget *parent) :
    QWidget(parent)
{
    searchWhatEdit = new ClearLineEdit(this);
    searchWhatEdit->setMinimumWidth(200);
    searchWhatEdit->setAttribute(Qt::WA_MacShowFocusRect, false);
    QMenu *lineEditMenu = new QMenu(searchWhatEdit);
    searchWhatEdit->setButtonMenu(FancyLineEdit::Left, lineEditMenu);
    searchWhatEdit->setButtonVisible(FancyLineEdit::Left, true);



    replaceByEdit = new ClearLineEdit(this);
    replaceByEdit->setMinimumWidth(200);
    replaceByEdit->setAttribute(Qt::WA_MacShowFocusRect, false);

    QToolButton *closeSearchPanel = new QToolButton(this);
    closeSearchPanel->setIcon(style()->standardPixmap(QStyle::SP_DockWidgetCloseButton));
    closeSearchPanel->setAutoRaise(true);
    connect(closeSearchPanel,SIGNAL(clicked()),this,SLOT(hide()));

    QToolButton *findNextButton = new QToolButton(this);
    findNextButton->setText(tr("Find next"));
    findNextButton->setAutoRaise(true);
    findNextButton->setShortcut(QKeySequence::FindNext);
    findNextButton->setToolTip(QString("%1 <font color=gray size=-1>%2</font>")
                               .arg(findNextButton->text())
                               .arg(findNextButton->shortcut().toString()));
    connect(findNextButton,SIGNAL(clicked()),SLOT(findNext()));

    QToolButton *findPrevButton = new QToolButton(this);
    findPrevButton->setText(tr("Find previous"));
    findPrevButton->setAutoRaise(true);
    findPrevButton->setShortcut(QKeySequence::FindPrevious);
    findPrevButton->setToolTip(QString("%1 <font color=gray size=-1>%2</font>")
                               .arg(findPrevButton->text())
                               .arg(findPrevButton->shortcut().toString()));
    connect(findPrevButton,SIGNAL(clicked()),SLOT(findPrev()));

    QToolButton *replaceAllButton = new QToolButton(this);
    replaceAllButton->setText(tr("Replace all"));
    replaceAllButton->setAutoRaise(true);
    connect(replaceAllButton,SIGNAL(clicked()),SIGNAL(replaceAll()));

    QToolButton *replaceButton = new QToolButton(this);
    replaceButton->setText(tr("Replace"));
    replaceButton->setAutoRaise(true);
    connect(replaceButton,SIGNAL(clicked()),SIGNAL(replace()));


    QToolButton *replaceAndFindButton = new QToolButton(this);
    replaceAndFindButton->setAutoRaise(true);
    replaceAndFindButton->setText(tr("Replace && Find"));
    connect(replaceAndFindButton,SIGNAL(clicked()),SIGNAL(replaceAndFind()));


    QGridLayout *searchPanelLayout = new QGridLayout;
    searchPanelLayout->setContentsMargins(0,0,0,0);
    searchPanelLayout->addWidget(new QLabel(tr("Find:"),this),0,0);
    searchPanelLayout->addWidget(searchWhatEdit,0,1);
    searchPanelLayout->addWidget(new QLabel(tr("Replace with:"),this),1,0);
    searchPanelLayout->addWidget(replaceByEdit,1,1);
    searchPanelLayout->addWidget(findPrevButton,0,2);
    searchPanelLayout->addWidget(replaceButton,1,2);
    searchPanelLayout->addWidget(findNextButton,0,3);
    searchPanelLayout->addWidget(replaceAndFindButton,1,3);
    //searchPanelLayout->addWidget(searchOptionsButton,0,4);
    searchPanelLayout->addWidget(replaceAllButton,1,4);
    QHBoxLayout *searchLayout = new QHBoxLayout;
    searchLayout->setContentsMargins(0,0,0,0);
    searchLayout->addLayout(searchPanelLayout);
    searchLayout->addWidget(closeSearchPanel,1,Qt::AlignTop | Qt::AlignRight);
    setLayout(searchLayout);

//    QMenu *searchOptionsMenu = new QMenu(this);
//    searchOptionsButton->setMenu(searchOptionsMenu);

    static const struct Option {
        const char *text;
        const char *slot;
        const char *icon;
    } options[] = {
        {QT_TR_NOOP("Case Sensitive"),SLOT(setCaseSensitive(bool)),":/src/icons/casesensitively.png"},
        {QT_TR_NOOP("Whole Words Only"),SLOT(setWholeWord(bool)),":/src/icons/wholewords.png"},
        {QT_TR_NOOP("Use Regular Expressions"),SLOT(setRegularExpressions(bool)),":/src/icons/regexp.png"}
    };
    for (int i=0; i<3; ++i) {
        QAction *a = new QAction(options[i].text, this);
        a->setCheckable(true);
        a->setChecked(false);
        a->setIcon(QIcon(QLatin1String(options[i].icon)));
        connect(a, SIGNAL(triggered(bool)), this, options[i].slot);
        lineEditMenu->addAction(a);
    }

    m_caseSensitive = false;
    m_wholeWord = false;
    m_useRegularExpressions = false;
    findFlagsChanged();
}

void SearchPanel::setCaseSensitive(bool b)
{
    m_caseSensitive = b;
    findFlagsChanged();
}

void SearchPanel::setWholeWord(bool b)
{
    m_wholeWord = b;
    findFlagsChanged();
}

void SearchPanel::setRegularExpressions(bool b)
{
    m_useRegularExpressions = b;
    findFlagsChanged();
}

void SearchPanel::startSearch(const QString &searchWhat)
{
    if (!searchWhat.isEmpty())
        searchWhatEdit->setText(searchWhat);
    else
        searchWhatEdit->setFocus();
    searchWhatEdit->selectAll();
}

void SearchPanel::findNext()
{
    emit find(true);
}

void SearchPanel::findPrev()
{
    emit find(false);
}

QPixmap SearchPanel::pixmapForFindFlags()
{
    static const QPixmap casesensitiveIcon = QPixmap(QSL(":/src/icons/casesensitively.png"));
    static const QPixmap regexpIcon = QPixmap(QSL(":/src/icons/regexp.png"));
    static const QPixmap wholewordsIcon = QPixmap(QSL(":/src/icons/wholewords.png"));

    int width = 0;
    if (m_caseSensitive) width += 6;
    if (m_wholeWord) width += 6;
    if (m_useRegularExpressions) width += 6;
    if (width > 0) --width;
    QPixmap pixmap(width, 17);
    pixmap.fill(Qt::transparent);
    QPainter painter(&pixmap);
    int x = 0;

    if (m_caseSensitive) {
        painter.drawPixmap(x - 6, 0, casesensitiveIcon);
        x += 6;
    }
    if (m_wholeWord) {
        painter.drawPixmap(x - 6, 0, wholewordsIcon);
        x += 6;
    }
    if (m_useRegularExpressions) {
        painter.drawPixmap(x - 6, 0, regexpIcon);
    }
    return pixmap;
}

void SearchPanel::findFlagsChanged()
{
    if (!m_caseSensitive && !m_wholeWord && !m_useRegularExpressions) {
        QPixmap pixmap(17, 17);
        pixmap.fill(Qt::transparent);
        QPainter painter(&pixmap);
        const QPixmap mag = QPixmap(QSL(":/src/icons/magnifier.png"));
        painter.drawPixmap(0, (pixmap.height() - mag.height()) / 2, mag);
        searchWhatEdit->setButtonPixmap(FancyLineEdit::Left, pixmap);
    } else {
        searchWhatEdit->setButtonPixmap(FancyLineEdit::Left, pixmapForFindFlags());
    }
}

const QString SearchPanel::replaceBy()
{
    return replaceByEdit->text();
}

const QString SearchPanel::searchWhat()
{
    return searchWhatEdit->text();
}


