/**
 * \file stringroutines.cpp
 * Various string functions
 *
 * \b Project: Qoobar
 * \author Alex Novichkov
 * \date 11 Aug 2011
 *
 * Copyright (C) 2011  Alex Novichkov
 *
 * This file is part of Qoobar.
 *
 * Qoobar is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Qoobar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "stringroutines.h"
#include <QStringList>
#include <QTextCodec>
#include <QtDebug>
#include "qoobarglobals.h"
#include "application.h"

QString replaceWinChars(const QString &source, const QString &byWhat, bool useSmartReplacing)
{
    QString s = source;
    static const struct ReplaceParams {
        const char *what;
        const char *byWhat;
    } params[] = {
        {"\\",0},
        {":",0},
        {"*",0},
        {"|",0},
        {"?",0},
        {"/",0},
        {"\"","''"},
        {"<","("},
        {">",")"}
    };
    for (int i=0; i<9; ++i) {
        if (useSmartReplacing && params[i].byWhat)
            s.replace(params[i].what,params[i].byWhat);
        else
            s.replace(params[i].what,byWhat);
    }
    return s;
}

bool isLatin(const QChar &c)
{
    ushort cu=c.unicode();
    return ((cu>=0x00c0 && cu<=0x024f) || (cu>=0x2c60 && cu<=0x2c7f)
            || (cu>=0x1e00 && cu<=0x1eff));
}

QString removeDiacritics(const QString &source)
{
    QString s = source;

    for(int charPos=0; charPos<s.length(); ++charPos) {
        QChar c=s.at(charPos);
        if (isLatin(c)) {//only latin chars
            QString decomposited=c.decomposition();
            if (!decomposited.isEmpty()) s.replace(charPos,1,decomposited.at(0));
        }
    }
    static const Rus stillToReplace[] = {
        {0x142,"l"},        {0x141,"L"},
        {0x153,"oe"},        {0x152,"OE"},
        {0xfe,"th"},        {0xde,"TH"},
        {0xf8,"o"},        {0xd8,"O"},
        {0xf0,"dh"},        {0xd0,"DH"},
        {0xe6,"ae"},        {0xc6,"AE"},
        {0xdf,"ss"},
        {0x131,"i"}
    };
    for (int i=0; i<14; ++i)
        s.replace(QChar(stillToReplace[i].index),QString::fromStdString(stillToReplace[i].l));
    return s;
}

QString changeCase(const QString &source,const Case::Case ca)
{
    if (ca==Case::AllSmall) return source.toLower();
    if (ca==Case::AllCaps)  return source.toUpper();

    QString s=source.toLower();
    int pos=0;
    QRegExp re(QSL("(\\b\\w)")); //beginning of the word
    while (pos>=0) {
        pos=s.indexOf(re,pos);
        if (pos>=0) {
            while (!s.at(pos).isLetter()) pos++;
            if (pos==s.length()) pos=-1;
            if (pos>=0) {
                s.replace(pos,1,s.at(pos).toUpper());
                if (ca==Case::FirstCaps) break;
                pos++;
            }
        }
    }
    if (ca==Case::EveryFirstCaps) {//apostrophes
        int pos=0;
        while (pos>=0) {
            pos=s.indexOf(QLS("'"),pos);
            if (pos>0 && pos<s.length()-1) {
                s.replace(pos+1,1,s.at(pos+1).toLower());
                pos++;
            }
            else if (pos==0) pos++;
            else pos=-1;
        }
    }

    return s;
}

QString trim(const QString &s)
{
    return s.trimmed();
}

QString simplifyWhitespaces(const QString &s)
{
    return s.simplified();
}

QString transliterate(const QString &source)
{
    QString s = source;

    static const Rus capitalLetters[] = {
        {0x410,"A"},
        {0x411,"B"},
        {0x412,"V"},
        {0x413,"G"},
        {0x414,"D"},
        {0x415,"E"},
        {0x401,"E"},
        {0x417,"Z"},
        {0x418,"I"},
        {0x419,"J"},
        {0x41a,"K"},
        {0x41b,"L"},
        {0x41c,"M"},
        {0x41d,"N"},
        {0x41e,"O"},
        {0x41f,"P"},
        {0x420,"R"},
        {0x421,"S"},
        {0x422,"T"},
        {0x423,"U"},
        {0x424,"F"},
        {0x426,"C"},
        {0x429,"X"},
        {0x42a,"W"},
        {0x42b,"Y"},
        {0x42c,"J"}
    };

    static const Rus capitalMultipleLetters[] = {
        {0x416,"Zh"},
        {0x425,"Kh"},
        {0x427,"Ch"},
        {0x428,"Sh"},
        {0x42d,"Eh"},
        {0x42e,"Yu"},
        {0x42f,"Ya"}
    };

    static const Rus lowerCaseLetters[] = {
        {0x430,"a"},
        {0x431,"b"},
        {0x432,"v"},
        {0x433,"g"},
        {0x434,"d"},
        {0x435,"e"},
        {0x451,"e"},
        {0x436,"zh"},
        {0x437,"z"},
        {0x438,"i"},
        {0x439,"j"},
        {0x43a,"k"},
        {0x43b,"l"},
        {0x43c,"m"},
        {0x43d,"n"},
        {0x43e,"o"},
        {0x43f,"p"},
        {0x440,"r"},
        {0x441,"s"},
        {0x442,"t"},
        {0x443,"u"},
        {0x444,"f"},
        {0x445,"kh"},
        {0x446,"c"},
        {0x447,"ch"},
        {0x448,"sh"},
        {0x449,"x"},
        {0x44a,"w"},
        {0x44b,"y"},
        {0x44c,"j"},
        {0x44d,"eh"},
        {0x44e,"yu"},
        {0x44f,"ya"},
    };

    for (int i=0; i<7; ++i)
        s.replace(QRegExp(QString("%1([%2-%3%4])")
                          .arg(QChar(capitalMultipleLetters[i].index))
                          .arg(QChar(0x430))
                          .arg(QChar(0x44f))
                          .arg(QChar(0x451))),
                  QString("%1\\1").arg(QString::fromStdString(capitalMultipleLetters[i].l)));
    for (int i=0; i<33; ++i)
        s.replace(QChar(lowerCaseLetters[i].index),
                  QString::fromStdString(lowerCaseLetters[i].l));
    for (int i=0; i<26; ++i)
        s.replace(QChar(capitalLetters[i].index),
                  QString::fromStdString(capitalLetters[i].l));
    for (int i=0; i<7; ++i)
        s.replace(QChar(capitalMultipleLetters[i].index),
                  QString::fromStdString(capitalMultipleLetters[i].l).toUpper());
    return s;
}

#include <QProcess>
#include <QMessageBox>
QString changeEncoding(const QString &source)
{

    QTextCodec *codec = 0;

    if (!programInstalled(QSL("enca"),0))
        return source;

    QStringList arguments;
    arguments << QSL("-g") << QSL("-m") << QSL("-L") << App->encaGuessLanguage;

    QProcess p;
    p.start(QSL("enca"),arguments);
    if (p.waitForStarted()) {
        p.write(source.toLatin1());
        p.closeWriteChannel();
        if (p.waitForFinished()) {
            QByteArray b=p.readAll();
            if (!b.isEmpty())
                codec = QTextCodec::codecForName(b.simplified().split(' ').at(0));
        }
    }
    if (codec) return codec->toUnicode(source.toLatin1());
    return source;
}


Operated::Operated(const QString &type)
    : m_type(type) { }

QString Operated::operator()(const QString &s)
{
    return operatedString(s, m_type);
}


QString operatedString(const QString &s, const QString &type)
{

    if (type=="simplify") return simplifyWhitespaces(s);
    if (type=="diacritics") return removeDiacritics(s);
    if (type=="transliterate") return transliterate(s);
    if (type=="encoding") return changeEncoding(s);

    if (type=="lower") return changeCase(s,Case::AllSmall);
    if (type=="upper") return changeCase(s,Case::AllCaps);
    if (type=="firstUp") return changeCase(s,Case::FirstCaps);
    if (type=="everyUp") return changeCase(s,Case::EveryFirstCaps);
    return s;
}
