/**
 * \file treewidget.cpp
 * QTreeWidget with some slots
 *
 * \b Project: Qoobar
 * \author Alex Novichkov
 * \date 11 Aug 2011
 *
 * Copyright (C) 2011  Alex Novichkov
 *
 * This file is part of Qoobar.
 *
 * Qoobar is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Qoobar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "treewidget.h"
#include "qoobarglobals.h"
#include "application.h"
#include "columnsdialog.h"
#ifdef HAVE_QT5
#include <QtWidgets>
#include <QtConcurrent/QtConcurrent>
#else
#include <QtGui>
#endif
#include "tab.h"
#include "tagsreaderwriter.h"
#include "enums.h"

int LENGTH = TAGSCOUNT;
int FILENAME = TAGSCOUNT+1;
int SAVEICON = TAGSCOUNT+2;
int REPLAYGAIN = TAGSCOUNT+3;
int IMAGE = TAGSCOUNT+4;

const int COLUMNCOUNT = TAGSCOUNT+5;

TreeWidget::TreeWidget(Tab *parent) :
    QTreeWidget(parent)
{
    tab=parent;
    saveIcon = qApp->style()->standardIcon(QStyle::SP_DialogSaveButton);
    replayGainIcon = QIcon(QSL(":/src/icons/replaygain.png"));
    imageIcon = QIcon(QSL(":/src/icons/image.png"));
    setMouseTracking(true);
    setRootIsDecorated(false);
    setAllColumnsShowFocus(true);
    setAlternatingRowColors(true);
    setSelectionMode(QAbstractItemView::ExtendedSelection);
    setUniformRowHeights(true);

#ifdef Q_OS_MAC
    setFrameStyle(QFrame::NoFrame | QFrame::Plain);
    //setAttribute(Qt::WA_MacShowFocusRect, false);
    setAutoFillBackground(true);
#endif

    setHeaderLabels(QVector<QString>(COLUMNCOUNT,QString()).toList());

    //by default the last column containing the "save" icon is placed in the first position.
    header()->moveSection(TRACKNUMBER,0);//Track number is in the second pos.
    header()->moveSection(FILENAME,1);
    header()->moveSection(SAVEICON,0);

    if (!App->columns160.isEmpty()) {
        header()->restoreState(App->columns160);
    }
#ifdef Q_OS_LINUX
    header()->setMinimumSectionSize(fontMetrics().width(QSL("5555")));
#endif
    header()->SETSECTIONRESIZEMODE(TRACKNUMBER, QHeaderView::ResizeToContents); //No.
    header()->SETSECTIONRESIZEMODE(LENGTH, QHeaderView::ResizeToContents); //length
    header()->SETSECTIONRESIZEMODE(SAVEICON, QHeaderView::ResizeToContents); //saved
    header()->SETSECTIONRESIZEMODE(REPLAYGAIN, QHeaderView::ResizeToContents); //saved
    header()->SETSECTIONRESIZEMODE(IMAGE, QHeaderView::ResizeToContents); //saved
    header()->setStretchLastSection(false);
    header()->setContextMenuPolicy(Qt::ActionsContextMenu);

    connect(header(),SIGNAL(geometriesChanged()),SLOT(updateTreeHeaderGeometry()));
    connect(header(),SIGNAL(sectionMoved(int,int,int)),SLOT(updateTreeHeaderGeometry()));
    connect(header(),SIGNAL(sectionResized(int,int,int)),SLOT(updateTreeHeaderGeometry()));



    QAction *a=new QAction(this);
    connect(a,SIGNAL(triggered()),this,SLOT(adjustDisplayedTags()));
    header()->addAction(a);

    uFont=qApp->font();
    bFont=uFont;
    bFont.setBold(true);
    currentLine=-1;
    mapper = new QSignalMapper(this);
    connect(mapper,SIGNAL(mapped(int)), this, SIGNAL(moveToTab(int)));
}

void TreeWidget::retranslateUi()
{
    for (int i=0; i<TAGSCOUNT; ++i)
        headerItem()->setText(i,tagByID(i));
    headerItem()->setText(LENGTH,tr("Length"));
    headerItem()->setText(TRACKNUMBER,tr("No."));
    headerItem()->setText(FILENAME,tr("File"));
    headerItem()->setText(REPLAYGAIN,tr("RG"));
    headerItem()->setText(IMAGE,tr("Img"));
    header()->actions().at(0)->setText(tr("Adjust displayed columns..."));
}

void TreeWidget::adjustDisplayedTags()
{
    ColumnsDialog dialog(this);
    dialog.exec();
}

void TreeWidget::markRow(int row,const QFont &f)
{
    if (row<0) return;

    QTreeWidgetItem *item=topLevelItem(row);
    if (item) {
        for (int i=0; i<item->columnCount(); ++i)
            item->setFont(i,f);
    }
}

void TreeWidget::editingDone()
{
    markRow(currentLine,uFont);
}

void TreeWidget::synchronizeLine(int curRow)
{
    markRow(curRow,bFont);
    markRow(currentLine,uFont);
    currentLine=curRow;
    scrollToItem(topLevelItem(curRow), QAbstractItemView::PositionAtCenter);
    repaint();
}

void TreeWidget::toggleFilesColumn(int column,int status)
{
    header()->setSectionHidden(column,status==HIDDEN);
}

void TreeWidget::updateTreeHeaderGeometry()
{
    App->columns160=header()->saveState();
}

void TreeWidget::updateFileSaveStatus(int row, bool wasChanged)
{
    const bool wC = !topLevelItem(row)->icon(SAVEICON).isNull();
    if (wC == wasChanged) return;
    topLevelItem(row)->setIcon(SAVEICON,wasChanged ? saveIcon : QIcon());
}

void TreeWidget::updateRow(int row, const Tag &tag, const QList<int> &rows)
{
    if (row<0 || row>=topLevelItemCount()) return;

    if (rows.isEmpty()) {
        for (int j=0; j<TAGSCOUNT; ++j)
            topLevelItem(row)->setText(j,tag.tag(j).simplified());
    }
    else {
        Q_FOREACH (const int &j, rows) {
            if (j<TAGSCOUNT) topLevelItem(row)->setText(j,tag.tag(j).simplified());
        }
    }
    //topLevelItem(row)->setIcon(REPLAYGAIN,tag.replayGainInfoIsEmpty() ? QIcon():replayGainIcon);
    //updateFileSaveStatus(row,tag.wasChanged());
}

void TreeWidget::updateRG(int row, const Tag &tag)
{
    if (row<0 || row>=topLevelItemCount()) return;
    const bool wC = topLevelItem(row)->icon(REPLAYGAIN).isNull();
    if (wC == tag.replayGainInfoIsEmpty()) return;
    topLevelItem(row)->setIcon(REPLAYGAIN,tag.replayGainInfoIsEmpty() ? QIcon():replayGainIcon);
}

void TreeWidget::updateImage(int row, bool imageEmpty)
{
    if (row<0 || row>=topLevelItemCount()) return;
    const bool wC = topLevelItem(row)->icon(IMAGE).isNull();
    if (wC == imageEmpty) return;
    topLevelItem(row)->setIcon(IMAGE,imageEmpty ? QIcon():imageIcon);
}

void TreeWidget::updateFileName(int index, const QString &newFileName)
{
    topLevelItem(index)->setText(FILENAME, newFileName);
}

QTreeWidgetItem *getItem(const Tag &tag)
{
    QStringList list=QVector<QString>(COLUMNCOUNT,QString()).toList();
    for (int i=0; i<TAGSCOUNT; ++i)
        list[i] = tag.tag(i).simplified();
    list[LENGTH]=formatLength(tag.length());
    list[FILENAME]=tag.fileNameExt();

    QTreeWidgetItem *item = new QTreeWidgetItem(list);

    item->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
    item->setIcon(FILENAME,QIcon(tag.icon()));
    item->setIcon(SAVEICON,tag.wasChanged()?qApp->style()->standardIcon(QStyle::SP_DialogSaveButton):QIcon());
    item->setIcon(REPLAYGAIN,tag.replayGainInfoIsEmpty() ? QIcon():QIcon(QSL(":/src/icons/replaygain.png")));
    item->setIcon(IMAGE,tag.imageIsEmpty() ? QIcon():QIcon(QSL(":/src/icons/image.png")));
    return item;
}

void TreeWidget::addRows(const QList<Tag> *tags)
{
    QList<QTreeWidgetItem *> list = QtConcurrent::blockingMapped(*tags, getItem);
    addTopLevelItems(list);
    updateSelection(tags->size());
}

void TreeWidget::updateSelection(int select)
{
    if (select<=0) return;
    if (select<100) {
        blockSignals(true);
        clearSelection();
        int count=topLevelItemCount();
        for (int i=count-select; i<count-1; ++i)
            topLevelItem(i)->setSelected(true);
        blockSignals(false);
        topLevelItem(count-1)->setSelected(true);
    }
}

void TreeWidget::contextMenuEvent(QContextMenuEvent *event)
{
    QMenu *menu = new QMenu(this);
    menu->insertActions(0,this->actions());

    if (tab) {
        QAction *moveToTabAct = new QAction(tr("Move to the tab..."),menu);
        moveToTabAct->setEnabled(this->selectionModel()->hasSelection());
        QMenu *moveToTabMenu = new QMenu(this);
        QMap<int, QString> tabNames = tab->allTabsNames();
        tabNames.insert(-1,tr("New tab"));
        Q_FOREACH (int tabID, tabNames.keys()) {
            QAction *a = new QAction(tabNames.value(tabID),moveToTabMenu);
            moveToTabMenu->insertAction(0,a);
            connect(a, SIGNAL(triggered()), mapper, SLOT(map()));
            mapper->setMapping(a, tabID);
        }
        moveToTabAct->setMenu(moveToTabMenu);
        menu->insertAction(0,moveToTabAct);

        if (tab->allSelectedFilesAreMP3()) {
            QAction *a = new QAction(tr("MP3 tag types..."),menu);
            connect(a, SIGNAL(triggered()), this, SIGNAL(showMP3TagsDialog()));
            menu->insertAction(0,a);
        }
    }

    menu->exec(event->globalPos());
    menu->deleteLater();
}

